/*
 * Reads a polyhedron file on stdin with rationals and outputs 
 * a canonical form with normalized rhs and sorted constraints.
 * 
 * David Bremner. bremner@cs.mcgill.ca
 *
 */

static char rcsid[]="$Id: rat2float.c 2343 2006-04-04 12:34:35Z bremner $";

#ifndef LRSMP
#error This file only compiles with LRSMP
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <float.h>
#include <errno.h>
#include "lrsmp.h"



typedef  lrs_mp integer_t;
#define MP_DIGITS 1000L


#include "rat2canon.h"


static int *rows=NULL;
static long int m,n;
static  lrs_mp **An;
static  lrs_mp **Ad;

int cmprow(const void *i_p,const void *j_p){
  int i=*(int *)i_p;
  int j=*(int *)j_p;
  int cval;
  int k=0;
  /* the trick here is that the denominators are always positive */
  while( k< n && 
	 ( cval=comprod(An[i][k],Ad[j][k],An[j][k],Ad[i][k])) == 0){
    k++;
  }
  return cval;
}

int main(argc,argv)
	 int argc;
	 char **argv;
{
  int i,j;
  char format[BUFSIZ];
  char *ok[]={ "integer", "rational" };
  char *type;
  char buf[BUFSIZ];

  process_args(argc,argv,DOCSTRING);
  lrs_mp_init (MP_DIGITS,stdin,stdout);   
  scan_for_begin(&m,&n,&type);
  check_type(type,  ok, 2);
  
  An=calloc(m,sizeof(lrs_mp*));
  Ad=calloc(m,sizeof(lrs_mp*));
  rows=calloc(m,sizeof(int));


  for (i=0; i<m; i++){
    rows[i]=i;
    An[i]=calloc(n,sizeof(lrs_mp));
    Ad[i]=calloc(n,sizeof(lrs_mp));
  }

  printf("%ld %ld rational\n",m,n);
     
  
  for (i=0;i<m;i++)
    {

  /* read in numbers */

      int looking=1;

      lrs_mp num0,den0;

      for(j=0;j<n;j++)
	{
	  char *p;
	  
	  integer_t num,den;
	  double out;

	  readrat(num,den);

	  if (looking==1 && !zero(num)){
	    copy(num0,num);

	    if (negative(num)) {
	      changesign(num0);
	      itomp(-1,An[i][j]);
	    } else {
	      itomp(1,An[i][j]);
	    }

	    copy(den0,den);
	    itomp(1,Ad[i][j]);


	    looking=0;
	  } else {

	    if (zero(num)) {
	      itomp(0,An[i][j]);
	      itomp(1,Ad[i][j]);
	    } else {
	      lrs_mp outnum,outden;
	      
	      if (!zero(num0)){
		divrat(num,den,num0,den0,An[i][j],Ad[i][j]);
	      } else {
		copy(An[i][j],num);
		copy(Ad[i][j],den);
	      }
	    }

	  }
	}
    }

  {


    fgets(buf,BUFSIZ,stdin);  /* clean off last line */
  }
  qsort(rows,m,sizeof(int), cmprow);

  for (i=0; i< m; i++){
    for(j=0; j<n; j++){
      prat("",An[rows[i]][j],Ad[rows[i]][j]);
    }
    fputs("\n",stdout);
  }
  
  while (fgets(buf,BUFSIZ,stdin) !=NULL )
    {
      fputs(buf,stdout);
    }
}






